#include "CFChatArea.H"
#include "StrUtil.H"

#include "CFColors.H"			//BEM

extern bool ROOM_ACTIVITY_OFF;		// BEM
extern bool CHAT_PMS_OFF;		// BEM


CFChatArea::CFChatArea(unsigned int wdth, unsigned int hght):
		TextArea(wdth,hght, 1024, true,_stripnl,false,4),
		_chatlog(NULL) {
	
}

CFChatArea::~CFChatArea() {
	if (_chatlog) {
		delete (_chatlog);
	}
}

int CFChatArea::enableLog(const string &fname) {
	FILE *fp;
	if ((fp=fopen(fname.c_str(),"a"))==NULL)
		return 1;
    _chatlog=new Logger(1,fp);
    (*_chatlog).pidOff();     
    (*_chatlog).timestampOn("%02d/%02m/%Y %H:%M:%S");
	return 0;
}

int CFChatArea::printMsg(const YahooMessage &msg) {
   	char
       	tmp[1024];
	bool
//		hascolor=scr->isColor();
		hascolor=true;

    if (msg.isLogout()) {
		printOfflineMsg();
	}
    else if (msg.isBuddyList()) {
		printList(msg.list(),"Here are your online friends:",
			 "(no friends online)");
    }
    else if (msg.isBuddyOnline()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_cyan);
		txt.style(s);
        sprintf(tmp,"%s is now online", msg.who().c_str());
		txt.setText(tmp);
		(*this)+=txt;
	}
    else if (msg.isBuddyOffline()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_cyan);
		txt.style(s);
        sprintf(tmp,"%s is now offline", msg.who().c_str());
		txt.setText(tmp);
		(*this)+=txt;
	}
    else if (msg.isBuddyAway()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_cyan);
		txt.style(s);
        sprintf(tmp,"%s is now away", msg.who().c_str());
		txt.setText(tmp);
		(*this)+=txt;
	}
    else if (msg.isBuddyBack()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_cyan);
		txt.style(s);
        sprintf(tmp,"%s is back", msg.who().c_str());
		txt.setText(tmp);
		(*this)+=txt;
	}
    else if (msg.isRoomEnter()) {
		Text
			txt;
		Char
			s;

		if (ROOM_ACTIVITY_OFF)		// BEM
			return 0;		// BEM

		if (hascolor) {
			s.setFore(ROOM_ACTIVITY_COLOR);
			s.tBold();
			txt.style(s);
        	sprintf(tmp,"%s entered the room", msg.who().c_str());
		}
		else {
        	sprintf(tmp,"(  %s entered the room  )", msg.who().c_str());
		}
		txt.setText(tmp);
		(*this)+=txt;
    }
    else if (msg.isRoomLeave()) {
		Text
			txt;
		Char
			s;

		if (ROOM_ACTIVITY_OFF)		// BEM
			return 0;		// BEM

		if (hascolor) {
			s.setFore(ROOM_ACTIVITY_COLOR);
			s.tBold();
			txt.style(s);
        	sprintf(tmp,"%s left the room",msg.who().c_str());
		}
		else {
        	sprintf(tmp,"(  %s left the room  )", msg.who().c_str());
		}
		txt.setText(tmp);
		(*this)+=txt;
    }
    else if (msg.isRoomList()) {
        Text
            txt;
		Char
			s;

		s.setFore(cc_green);
        txt.style(s);
        txt.setText(" ");
		(*this)+=txt;
		(*this)+=txt;
        sprintf(tmp,"You are in %s",msg.room().c_str());
        txt.setText(tmp);
		(*this)+=txt;
		s.setFore(cc_white);
        txt.style(s);
        txt.setText(msg.desc());
		(*this)+=txt;
        txt.setText(" ");
		(*this)+=txt;
    }
    else if (msg.isInvite()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_green);
		txt.style(s);
        sprintf(tmp,"%s", msg.msg().c_str());
		txt.setText(tmp);
		(*this)+=txt;
    }
    else if (msg.isUserStatus()) {
		Text
			who,
			txt;
		Char
			s;

		s.setFore(cc_red);
		who.style(s);
		who.setText(msg.who());
		s.setFore(cc_magenta);
		txt.style(s);
        sprintf(tmp," %s",msg.msg().c_str());
		txt.setText(tmp);
		who+=txt;
		s.setFore(cc_white);
		txt.style(s);
        sprintf(tmp," %s",msg.desc().c_str());
		txt.setText(tmp);
		who+=txt;
		(*this)+=who;
	}
    else if (msg.isSpeech()) {
        Text
            txt;
		Char
			s;

		if (hascolor)
        	s.tBold();
        if (msg.isSelf())
            s.setFore(cc_blue);
        else
            s.setFore(cc_red);
		txt.style(s);
        txt.setText(msg.who());
        sprintf(tmp,": %s\n",msg.msg().c_str());
        txt+=tmp;
		(*this)+=txt;
		if (_chatlog) {
        	(*_chatlog).log("%s: %s\n",msg.who().c_str(),msg.msg().c_str());
        	(*_chatlog).flush();
		}
    }
    else if (msg.isPM()) {
		Char
			s;
        Text
            who,
            what;
		StrUtil
			sutil;
		string
			message;

		if (CHAT_PMS_OFF)		// BEM
			return 0;		// BEM

		if (hascolor)
        	s.tBold();
        if (msg.isSelf())
            s.setFore(cc_blue);
        else
            s.setFore(cc_red);
		who.style(s);
        who.setText(msg.who());
		s.setFore(cc_yellow);
		what.style(s);
        what.setText(" [PM] ");
        who+=what;
		if (hascolor)
        	s.tBold();
		if ((msg.msg()[0]==':') &&
			(!sutil.isSmiley(msg.msg()))) {
			const char *ptr;
        	s.setFore(cc_magenta);
			ptr=msg.msg().c_str();
			message=ptr+1;
		}
		else {
        	s.setFore(cc_white);
			message=msg.msg();
		}
		what.style(s);
		what.setText(message);
        who+=what;
		(*this)+=who;
		if (_chatlog) {
        	(*_chatlog).log("%s: [PM]  %s\n",msg.who().c_str(),msg.msg().c_str());
        	(*_chatlog).flush();
		}
    }
    else if ((msg.isThought()) || (msg.isEmotion())) {
		Char
			s;
        Text
            who,
            what;

		if (hascolor)
        	s.tBold();
        if (msg.isSelf())
            s.setFore(cc_blue);
        else
            s.setFore(cc_red);
		who.style(s);
        who.setText(msg.who());
		if (hascolor)
			s.tBold();
        s.setFore(ROOM_EMOTE_COLOR);
        what.style(s);
        sprintf(tmp," %s\n",msg.msg().c_str());
        what.setText(tmp);
        who+=what;
		(*this)+=who;
		if (_chatlog) {
        	(*_chatlog).log("%s: %s",msg.who().c_str(),msg.msg().c_str());
        	(*_chatlog).flush();
		}
    }
    else if (msg.isDice()) {
		Text
			txt;
		Char
			s;

		s.setFore(cc_cyan);
		txt.style(s);
        sprintf(tmp,"%s\n",msg.msg().c_str());
		txt.setText(tmp);
		(*this)+=txt;
		if (_chatlog) {
        	(*_chatlog).log("%s: %s",msg.who().c_str(),msg.msg().c_str());
        	(*_chatlog).flush();
		}
    }
    else if (msg.isAd()) {
		// hehe FUCK the ads
	}
	else {
		return 1;
	}
	return 0;
}

void CFChatArea::printOfflineMsg() {
	Text
		txt;
	txt.setText(" ");
	(*this)+=txt;
	(*this)+=txt;
	(*this)+=txt;
	txt.setText("<<< you are now offline >>>");
	(*this)+=txt;
}

void CFChatArea::printList(const StrList &ulist, const char *headermsg,
		const char *emptymsg) {
	unsigned int
		i=0;
	Char
		s;
	Text
		delim,
		who,
		txt;
	s.setFore(cc_cyan);
	if (headermsg) {
		s.tBold();
		txt.style(s);
		txt.setText(headermsg);
		(*this)+=txt;
		s.tBold();
	}
	txt.style(s);
	delim.style(s);
	who.style(s);
	delim.setText(", ");
	if (ulist.size())
		txt.setText(ulist[i++]);
	else if (emptymsg)
		txt.setText(emptymsg);
	while (i < ulist.size()) {
		txt+=delim;
		who.setText(ulist[i++]);
		txt+=who;
	}
	delim.setText(" ");
	(*this)+=txt;
	(*this)+=delim;
}
